"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const sqlite3orm_1 = require("sqlite3orm");
const sqlite3_connector_1 = require("./sqlite3-connector");
function debug(arg, ...args) {
    sqlite3_connector_1.Sqlite3Connector.debug(arg, ...args);
}
/*
 * The MetaModel factory and registry
 */
class MetaModelFactory {
    /**
     * Creates an instance of MetaModels.
     *
     */
    constructor() {
        if (!MetaModelFactory.models) {
            this.name = sqlite3_connector_1.SQLITE3_CONNECTOR_NAME;
            this.mapNameToMetaModel = new Map();
            // initialize the 'singleton'
            MetaModelFactory.models = this;
        }
        return MetaModelFactory.models;
    }
    /**
     * get a MetaModel for a lb4 model definition
     *
     * @param modelName
     * @param lbModelDef
     * @param [recreate]
     */
    // TODO: getMetaModel for the CRUDConnector is not implemented yet
    /* istanbul ignore next */
    getMetaModel(modelName, lbModelDef, recreate) {
        throw new Error(`getMetaModel is not implemented yet`);
    }
    /**
     * get a MetaModel for a juggler model definition
     *
     * @param modelName
     * @param lbModelDef
     * @param [recreate]
     */
    getMetaModelFromJuggler(modelName, lbModelDef, recreate) {
        let metaModelRef = this.mapNameToMetaModel.get(modelName);
        if (metaModelRef) {
            if (!recreate && lbModelDef === metaModelRef.lbModelDef) {
                return metaModelRef.metaModel;
            }
            this.destroyMetaModel(modelName);
        }
        const settings = lbModelDef.settings || {};
        const modelOpts = (settings[this.name] || {});
        debug(`registering model '${modelName}'`);
        metaModelRef = { metaModel: new sqlite3orm_1.MetaModel(modelName), lbModelDef };
        const tableOpts = {
            name: modelOpts.tableName || this.dbName(modelName),
            withoutRowId: modelOpts.withoutRowId
        };
        // properties:
        const properties = lbModelDef.properties || {};
        const mapKeyToColName = new Map();
        Object.keys(properties)
            .filter((propName) => properties.hasOwnProperty(propName))
            .forEach((propName) => {
            const property = properties[propName];
            if (property.id && property.generated) {
                tableOpts.autoIncrement = true;
            }
            const propertyOpts = (property[this.name] || {});
            property[this.name] = propertyOpts;
            const metaProp = metaModelRef.metaModel.getOrAddProperty(propName);
            if (property.type &&
                (typeof property.type === 'function' ||
                    typeof property.type === 'string')) {
                metaProp.setPropertyType(property.type);
            }
            // applying defaults:
            if (metaProp.propertyType === sqlite3orm_1.PropertyType.UNKNOWN) {
                propertyOpts.isJson = true;
            }
            if (metaProp.propertyType === sqlite3orm_1.PropertyType.DATE &&
                // tslint:disable-next-line triple-equals
                propertyOpts.dateInMilliSeconds == undefined) {
                propertyOpts.dateInMilliSeconds = true;
            }
            const fieldOpts = {
                name: propertyOpts.columnName || this.dbName(propName),
                dbtype: propertyOpts.dbtype,
                isJson: propertyOpts.isJson,
                dateInMilliSeconds: propertyOpts.dateInMilliSeconds,
                transform: propertyOpts.transform
            };
            metaModelRef.metaModel.setPropertyField(propName, !!property.id, fieldOpts);
            mapKeyToColName.set(propName, fieldOpts.name);
        });
        metaModelRef.metaModel.init(tableOpts);
        Object.keys(properties).forEach((propName) => {
            const property = properties[propName];
            if (!property[this.name] || property[this.name].transform) {
                return;
            }
            const metaProp = metaModelRef.metaModel.getOrAddProperty(propName);
            property[this.name].transform = metaProp.transform;
        });
        // indexes:
        if (typeof settings.indexes === 'object') {
            this.addIndexDefinitions(metaModelRef, settings.indexes);
        }
        // foreignKeys:
        if (typeof settings.foreignKeys === 'object') {
            this.addForeignKeyDefinitions(metaModelRef, settings.foreignKeys);
        }
        this.mapNameToMetaModel.set(metaModelRef.metaModel.name, metaModelRef);
        return metaModelRef.metaModel;
    }
    addIndexDefinitions(metaModelRef, indexes) {
        const metaModel = metaModelRef.metaModel;
        const table = metaModel.table;
        Object.keys(indexes)
            .filter((indexName) => indexes.hasOwnProperty(indexName))
            .forEach((indexName) => {
            const indexDef = indexes[indexName];
            if (typeof indexDef.columns === 'string') {
                // using MySql index definition:
                const columns = indexDef.columns.split(',');
                const idxDef = new sqlite3orm_1.IDXDefinition(indexName, indexDef.kind && indexDef.kind === 'unique');
                columns.forEach((colName) => {
                    idxDef.fields.push({ name: colName.trim() });
                });
                table.addIDXDefinition(idxDef);
            }
            else {
                // standard or shortened form:
                const keys = typeof indexDef.keys === 'object' ? indexDef.keys : indexDef;
                // tslint:disable-next-line no-unnecessary-initializer
                let isUnique = undefined;
                if (indexDef.options &&
                    // tslint:disable-next-line triple-equals
                    typeof indexDef.options.unique != undefined) {
                    isUnique = !!indexDef.options;
                }
                const idxDef = new sqlite3orm_1.IDXDefinition(indexName, isUnique);
                Object.keys(keys).forEach((propName) => {
                    const metaProp = metaModel.hasProperty(propName);
                    /* istanbul ignore if */
                    if (!metaProp) {
                        throw new Error(`property '${propName}' not defined for model '${metaModel.name}' but referenced in index definition '${indexName}'`);
                    }
                    idxDef.fields.push({
                        name: metaProp.field.name,
                        desc: keys[propName] === -1 ? true : false
                    });
                });
                table.addIDXDefinition(idxDef);
            }
        });
    }
    addForeignKeyDefinitions(metaModelRef, foreignKeys) {
        const metaModel = metaModelRef.metaModel;
        const table = metaModel.table;
        Object.keys(foreignKeys)
            .filter((constraintName) => foreignKeys.hasOwnProperty(constraintName))
            .forEach((constraintName) => {
            const properties = foreignKeys[constraintName].properties.split(',');
            const refColumns = foreignKeys[constraintName].refColumns.split(',');
            const refTable = foreignKeys[constraintName].refTable;
            if (foreignKeys[constraintName].name) {
                constraintName = foreignKeys[constraintName].name;
            }
            /* istanbul ignore if */
            if (properties.length !== refColumns.length) {
                throw new Error(`invalid foreign key definition '${constraintName}': number of items in 'properties' and 'refColumns' must be equal`);
            }
            const fkDef = new sqlite3orm_1.FKDefinition(constraintName, refTable);
            for (let i = 0; i < refColumns.length; i++) {
                properties[i] = properties[i].trim();
                const metaProp = metaModel.hasProperty(properties[i]);
                /* istanbul ignore if */
                if (!metaProp) {
                    throw new Error(`property '${properties[i]}' not defined for model '${metaModel.name}' but referenced in foreign key definition '${constraintName}'`);
                }
                fkDef.fields.push({
                    name: metaProp.field.name,
                    foreignColumnName: refColumns[i] ? refColumns[i].trim() : ''
                });
            }
            table.addFKDefinition(fkDef);
        });
    }
    /**
     * destroy a MetaModel by name
     *
     * @param modelName
     */
    destroyMetaModel(modelName) {
        const metaModelRef = this.mapNameToMetaModel.get(modelName);
        /* istanbul ignore if */
        if (!metaModelRef) {
            return;
        }
        this.mapNameToMetaModel.delete(modelName);
        metaModelRef.metaModel.destroy();
    }
    /**
     * destroy all models
     *
     */
    destroyAllMetaModels() {
        this.mapNameToMetaModel.forEach((metaModelRef) => {
            metaModelRef.metaModel.destroy();
        });
        this.mapNameToMetaModel = new Map();
    }
    modelNames() {
        return Array.from(this.mapNameToMetaModel.keys());
    }
    /**
     * get the default table/column name for a given model/property name
     * @param name The name
     */
    dbName(modelOrProperty) {
        /* istanbul ignore if */
        if (!modelOrProperty) {
            return modelOrProperty;
        }
        return modelOrProperty.toLowerCase();
    }
}
exports.MetaModelFactory = MetaModelFactory;
//# sourceMappingURL=meta-model-factory.js.map