"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const _sg = require("strong-globalize");
const _dbg = require("debug");
const sqlite3orm_1 = require("sqlite3orm");
const meta_model_factory_1 = require("./meta-model-factory");
const g = new _sg();
exports.SQLITE3_CONNECTOR_NAME = 'sqlite3x';
const SQLITE3_CONNECTOR_DESCRIPTION = g.f('unofficial LoopBack connector for SQLite3');
const debug = _dbg('loopback:connector:sqlite3x');
class Sqlite3Connector {
    constructor(settings) {
        this.name = exports.SQLITE3_CONNECTOR_NAME;
        /* istanbul ignore if */
        if (Sqlite3Connector.debugEnabled) {
            debug(`${exports.SQLITE3_CONNECTOR_NAME}`);
            debug(`  ${SQLITE3_CONNECTOR_DESCRIPTION}`);
            debug(`  settings: %j`, settings);
            sqlite3orm_1.SqlDatabase.verbose();
        }
        this.settings = Sqlite3Connector.enrichInputSettings(settings || {});
        this.pool = new sqlite3orm_1.SqlConnectionPool();
        this.metaModels = new meta_model_factory_1.MetaModelFactory();
    }
    /* istanbul ignore next */
    static get debugEnabled() {
        return debug.enabled;
    }
    /* istanbul ignore next */
    static set debugEnabled(enabled) {
        debug.enabled = enabled;
    }
    /* istanbul ignore next */
    static debug(arg, ...args) {
        debug(arg, ...args);
    }
    /*************************************************************************************
     * Connector interface
     */
    /**
     * connect to Sqlite3 database pool
     *
     * @returns a void-promise
     */
    connect() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.pool.isOpen()) {
                return;
            }
            try {
                debug('connecting pool...');
                yield this.pool.open(this.settings.file, this.settings.mode, this.settings.poolMin, this.settings.poolMax, this.settings.dbSettings);
                debug('connected pool');
            }
            catch (err) {
                debug(`connecting pool failed: ${err}`);
                return Promise.reject(err);
            }
        });
    }
    /**
     * disconnect from Sqlite3 database pool
     *
     * @returns a void-promise
     */
    disconnect() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.pool.isOpen()) {
                return;
            }
            try {
                debug('disconnecting pool...');
                yield this.pool.close();
                debug('disconnected pool');
            }
            catch (err) /* istanbul ignore next */ {
                debug(`disconnecting pool failed: ${err}`);
                return Promise.reject(err);
            }
        });
    }
    /**
     * ping the database backend
     *
     * @returns a void-promise
     */
    ping() {
        const sql = 'PRAGMA user_version';
        return this.execSql(sql).then(() => { });
    }
    /**
     * get connecton from pool
     */
    getConnection() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.pool || !this.pool.isOpen()) {
                return Promise.reject(new Error(g.f('no pool connected')));
            }
            try {
                /* istanbul ignore if */
                if (this.pool.opening) {
                    // waiting for pool to open
                    yield this.pool.opening;
                }
                const res = yield this.pool.get();
                return res;
            }
            catch (err) /* istanbul ignore next */ {
                return Promise.reject(g.f('connecting failed'));
            }
        });
    }
    /**
     * begin transaction
     */
    begin() {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                debug('begin transaction...');
                const connection = yield this.getConnection();
                yield Sqlite3Connector.runDML(connection, `BEGIN DEFERRED TRANSACTION`);
                return connection;
            }
            catch (err) /* istanbul ignore next */ {
                debug(`begin transaction failed: ${err}`);
                return Promise.reject(err);
            }
        });
    }
    /**
     * execute sql on new connection from pool
     * release connection afterwards
     */
    execSql(sql, params) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const connection = yield this.getConnection();
                const res = yield this.runSQL(connection, sql, params);
                // release connection to pool
                try {
                    yield connection.close();
                }
                catch (_ignore) {
                }
                return res;
            }
            catch (err) {
                return Promise.reject(err);
            }
        });
    }
    /**
     * commit transaction
     *
     * @param connection
     */
    commit(connection) {
        return __awaiter(this, void 0, void 0, function* () {
            debug('commit transaction');
            let err;
            try {
                yield Sqlite3Connector.runDML(connection, `COMMIT TRANSACTION`);
            }
            catch (e) /* istanbul ignore next */ {
                err = e;
            }
            try {
                yield connection.close();
            }
            catch (_ignore) {
            }
            /* istanbul ignore if */
            if (err) {
                return Promise.reject(err);
            }
        });
    }
    /**
     * rollback transaction
     *
     * @param connection
     */
    rollback(connection) {
        return __awaiter(this, void 0, void 0, function* () {
            debug('rollback transaction');
            try {
                yield Sqlite3Connector.runDML(connection, `ROLLBACK TRANSACTION`);
            }
            catch (e) /* istanbul ignore next */ {
                // ignore
                // see: https://www.sqlite.org/lang_transaction.html
                // quote: If the transaction has already been rolled back automatically by
                // the error response, then the ROLLBACK command will fail with an error,
                // but no harm is caused by this.
            }
            try {
                yield connection.close();
            }
            catch (_ignore) {
            }
        });
    }
    static runDQL(conn, sql, params) {
        return conn.all(sql, params);
    }
    static runDML(conn, sql, params) {
        return conn.run(sql, params).catch((err) => {
            if (err && err.message.match(/UNIQUE constraint failed/i)) {
                err.message = `DUPLICATE: ${err.message}`;
            }
            return Promise.reject(err);
        });
    }
    runSQL(conn, sql, params) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                let res;
                const sqlType = sql.trimLeft().substring(0, 6).toUpperCase();
                if (sqlType === 'SELECT' || sqlType === 'PRAGMA') {
                    res = yield Sqlite3Connector.runDQL(conn, sql, params);
                }
                else {
                    res = yield Sqlite3Connector.runDML(conn, sql, params);
                }
                return Promise.resolve(res);
            }
            catch (err) /* istanbul ignore nothing */ {
                return Promise.reject(err);
            }
        });
    }
    // *************************************************************************************
    // model definitions
    // -------------------------------------------------------------------------------------
    /* istanbul ignore next */
    getMetaModel(modelName, lbModelDef, recreate) {
        throw new Error(`not implemented yet`);
        // return this.metaModels.getMetaModel(modelName, lbModelDef, recreate);
    }
    getMetaModelFromJuggler(modelName, lbModelDef, recreate) {
        return this.metaModels.getMetaModelFromJuggler(modelName, lbModelDef, recreate);
    }
    destroyMetaModel(modelName) {
        this.metaModels.destroyMetaModel(modelName);
    }
    destroyAllMetaModels() {
        this.metaModels.destroyAllMetaModels();
    }
    dbName(modelOrProperty) {
        return this.metaModels.dbName(modelOrProperty);
    }
    // *************************************************************************************
    // settings
    // -------------------------------------------------------------------------------------
    static enrichInputSettings(inputSettings) {
        const connectorSettings = Object.assign({}, inputSettings);
        connectorSettings.file = connectorSettings.file || sqlite3orm_1.SQL_MEMORY_DB_SHARED;
        connectorSettings.mode = connectorSettings.mode || sqlite3orm_1.SQL_OPEN_DEFAULT;
        connectorSettings.poolMin = connectorSettings.poolMin || 1;
        // tslint:disable triple-equals
        connectorSettings.poolMax =
            connectorSettings.poolMax == undefined ? 0 : connectorSettings.poolMax;
        connectorSettings.schemaName = connectorSettings.schemaName || 'main';
        connectorSettings.debug = Sqlite3Connector.debugEnabled;
        /* istanbul ignore else */
        if (!connectorSettings.dbSettings) {
            connectorSettings.dbSettings = {};
        }
        /* istanbul ignore else */
        if (!connectorSettings.dbSettings.journalMode) {
            connectorSettings.dbSettings.journalMode = 'WAL';
        }
        /* istanbul ignore else */
        if (!connectorSettings.dbSettings.busyTimeout) {
            connectorSettings.dbSettings.busyTimeout = 3000;
        }
        /* istanbul ignore else */
        if (!connectorSettings.dbSettings.readUncommitted) {
            connectorSettings.dbSettings.readUncommitted = 'FALSE';
        }
        /* istanbul ignore else */
        if (!connectorSettings.dbSettings.executionMode) {
            connectorSettings.dbSettings.executionMode = 'PARALLELIZE';
        }
        return connectorSettings;
    }
}
exports.Sqlite3Connector = Sqlite3Connector;
//# sourceMappingURL=sqlite3-connector.js.map