import { Callback, DataSource, Filter, PromiseOrVoid, PropertyDefinition, TransactionMixin } from 'loopback-datasource-juggler';
import { MetaModel, SqlConnectionPool, SqlDatabase, SqlRunResult } from 'sqlite3orm';
import { ParameterizedSQL, SQLConnector } from './lc-import';
import { Sqlite3ExecuteOptions } from './sqlite3-options';
import { Sqlite3AllSettings, Sqlite3Settings } from './sqlite3-settings';
export declare class Sqlite3JugglerConnector extends SQLConnector implements TransactionMixin {
    readonly name: string;
    private readonly connector;
    readonly settings: Sqlite3AllSettings;
    readonly pool: SqlConnectionPool;
    constructor(settings: Sqlite3Settings | Object);
    static debug(arg: any, ...args: any[]): void;
    /**
     * connect to Sqlite3 database pool
     *
     * @callback [cb] The callback function
     */
    connect(cb?: Callback<void>): PromiseOrVoid;
    /**
     * disconnect from Sqlite3 database pool
     *
     * @callback [cb] The callback function
     */
    disconnect(cb?: Callback<void>): PromiseOrVoid;
    /**
     * ping the database backend
     *
     * @callback [cb] The callback function
     */
    ping(cb?: Callback<void>): PromiseOrVoid;
    beginTransaction(isolationLevel: string, cb?: Callback): PromiseOrVoid<SqlDatabase>;
    beginTransaction(cb?: Callback): PromiseOrVoid;
    protected _beginTransaction(cb?: Callback): PromiseOrVoid<SqlDatabase>;
    commit(connection: SqlDatabase, cb?: Callback): PromiseOrVoid;
    rollback(connection: SqlDatabase, cb?: Callback): PromiseOrVoid;
    /**
     * Execute a SQL statement with given parameters
     *
     * @param sql The SQL statement
     * @param [params] An array of parameter values
     * @param [options] Options object
     * @callback [callback] The callback function
     */
    executeSQL(sql: string, params?: any[], options?: Sqlite3ExecuteOptions, cb?: Callback): PromiseOrVoid;
    executeSQL(sql: string, params?: any[], cb?: Callback): PromiseOrVoid;
    executeSQL(sql: string, cb?: Callback): PromiseOrVoid;
    protected _executeSQL(sql: string, params?: any[], options?: Sqlite3ExecuteOptions, cb?: Callback): PromiseOrVoid;
    promisifiedExecuteSql(sql: string, params?: any[], options?: Sqlite3ExecuteOptions): Promise<any[] | SqlRunResult>;
    getInsertedId(model: string, result?: SqlRunResult): number | undefined;
    getCountForAffectedRows(model: string, result?: SqlRunResult): number | undefined;
    getPlaceholderForValue(varName: string): string;
    buildInsertDefaultValues(model: string, data: any, options?: object): string;
    fromColumnValue(prop: PropertyDefinition, value: any): any;
    toColumnValue(prop: PropertyDefinition, value: any): any;
    /**
     * get the default table/column name for a given model/property name
     * @param name The name
     */
    dbName(modelOrProperty?: string): string | undefined;
    /**
     * escape/quote the table/column name
     * @param name The name
     */
    escapeName(dbName?: string): string | undefined;
    /**
     * escape a value
     * @param name The name
    escapeValue(value: any): any {
      return value;
    }
     */
    static buildLimit(filter: Filter): string | undefined;
    applyPagination(model: string, sql: ParameterizedSQL, filter: Filter): ParameterizedSQL;
    /**
     * Create the table for the given model
     * @param model The model name
     * @param  [cb] The callback function
     */
    createTable(model: string, cb?: Callback): PromiseOrVoid;
    /**
     * drop the table for the given model
     * @param model The model name
     * @param  [cb] The callback function
     */
    dropTable(model: string, cb?: Callback): PromiseOrVoid;
    /**
     * Perform autoupdate for the given models or all models
     *
     * @param [models] - The model name or array of model names
     * @param [cb] - The callback function
     */
    autoupdate(models?: string | string[], cb?: Callback): PromiseOrVoid;
    autoupdate(cb?: Callback): PromiseOrVoid;
    protected _autoupate(models?: string[], cb?: Callback): PromiseOrVoid;
    protected promisifiedAutoupdate(models?: string[]): Promise<void>;
    /**
     * Check if the models exist and are up-to-date
     *
     * @param [models] - The model name or array of model names
     * @param [cb] - The callback function
     */
    isActual(models?: string | string[], cb?: Callback): PromiseOrVoid<boolean>;
    isActual(cb?: Callback): PromiseOrVoid<boolean>;
    protected _isActual(models?: string[], cb?: Callback): PromiseOrVoid<boolean>;
    protected promisifiedIsActual(models?: string[]): Promise<boolean>;
    getMetaModel(modelName: string, recreate?: boolean): MetaModel;
    destroyMetaModel(modelName: string): void;
    destroyAllMetaModels(): void;
    modelNames(): string[];
}
/**
 *
 * Initialize the Sqlite3Connector for the given data source
 *
 * @param dataSource The dataSource
 * @callback [cb] The callback function
 */
export declare function initialize(dataSource: DataSource, cb?: Callback<void>): void;
