"use strict";
// tslint:disable no-implicit-dependencies no-string-literal
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const sqlite3orm_1 = require("sqlite3orm");
const lc_import_1 = require("./lc-import");
const sqlite3_connector_1 = require("./sqlite3-connector");
const utils_1 = require("./utils");
/* istanbul ignore next */
function debug(arg, ...args) {
    sqlite3_connector_1.Sqlite3Connector.debug(arg, ...args);
}
/*
 * wrapper class around Sqlite3CrudConnector to make juggler happy
 */
class Sqlite3JugglerConnector extends lc_import_1.SQLConnector {
    constructor(settings) {
        super(sqlite3_connector_1.SQLITE3_CONNECTOR_NAME, settings);
        this.name = sqlite3_connector_1.SQLITE3_CONNECTOR_NAME;
        this.connector = new sqlite3_connector_1.Sqlite3Connector(settings);
        this.settings = this.connector.settings;
    }
    get pool() {
        return this.connector.pool;
    }
    /* istanbul ignore next */
    static debug(arg, ...args) {
        debug(arg, ...args);
    }
    // *************************************************************************************
    //  connect, disconnect, ping, ...
    // -------------------------------------------------------------------------------------
    /**
     * connect to Sqlite3 database pool
     *
     * @callback [cb] The callback function
     */
    connect(cb) {
        return utils_1.callbackifyOrPromise(this.connector.connect(), cb);
    }
    /**
     * disconnect from Sqlite3 database pool
     *
     * @callback [cb] The callback function
     */
    disconnect(cb) {
        return utils_1.callbackifyOrPromise(this.connector.disconnect(), cb);
    }
    /**
     * ping the database backend
     *
     * @callback [cb] The callback function
     */
    ping(cb) {
        return utils_1.callbackifyOrPromise(this.connector.ping(), cb);
    }
    beginTransaction(isolationLevel, cb) {
        /* istanbul ignore if */
        if (typeof isolationLevel === 'function' && cb === undefined) {
            return this._beginTransaction(isolationLevel);
        }
        else {
            return this._beginTransaction(cb);
        }
    }
    _beginTransaction(cb) {
        return utils_1.callbackifyOrPromise(this.connector.begin(), cb);
    }
    commit(connection, cb) {
        return utils_1.callbackifyOrPromise(this.connector.commit(connection), cb);
    }
    rollback(connection, cb) {
        return utils_1.callbackifyOrPromise(this.connector.rollback(connection), cb);
    }
    executeSQL(sql, params, options, cb) {
        /* istanbul ignore if */
        if (typeof params === 'function' && cb === undefined &&
            options === undefined) {
            cb = params;
            return this._executeSQL(sql, undefined, undefined, cb);
        }
        else {
            /* istanbul ignore if */
            if (typeof options === 'function' && cb === undefined) {
                cb = options;
                return this._executeSQL(sql, params, undefined, cb);
            }
            else {
                return this._executeSQL(sql, params, options, cb);
            }
        }
    }
    _executeSQL(sql, params, options, cb) {
        return utils_1.callbackifyOrPromise(this.promisifiedExecuteSql(sql, params, options), cb);
    }
    promisifiedExecuteSql(sql, params, options) {
        return __awaiter(this, void 0, void 0, function* () {
            params = params || [];
            options = options || {};
            let res;
            try {
                const transaction = options && options.transaction;
                if (transaction && transaction.connection &&
                    transaction.connector === this) {
                    res = yield this.connector.runSQL(transaction.connection, sql, params);
                }
                else {
                    res = yield this.connector.execSql(sql, params);
                }
            }
            catch (err) {
                return Promise.reject(err);
            }
            return Promise.resolve(res);
        });
    }
    getInsertedId(model, result) {
        // the 'result.lastID' is the last inserted RowId
        const metaModel = this.getMetaModel(model);
        /* istanbul ignore else */
        if (result && metaModel.table.mapNameToIdentityField.size === 1) {
            const fld = metaModel.table.mapNameToIdentityField.values().next();
            if (fld.value.dbTypeInfo.typeAffinity === 'INTEGER') {
                // Only if the primary key is integer, it is an alias for the RowId.
                return result.lastID;
            }
        }
        // result.lastID is not the "inserted ID"
        return undefined;
    }
    getCountForAffectedRows(model, result) {
        return (result && result.changes) || 0;
    }
    getPlaceholderForValue(varName) {
        return ':' + varName;
    }
    buildInsertDefaultValues(model, data, options) {
        return 'DEFAULT VALUES';
    }
    fromColumnValue(prop, value) {
        /* istanbul ignore else */
        if (prop && prop[this.name] && prop[this.name].transform &&
            prop[this.name].transform.fromDB) {
            const v = prop[this.name].transform.fromDB(value); // using 'undefined' for NULL
            return v !== undefined ? v : this.settings.propertyValueForNULL;
        }
        /* istanbul ignore next */
        throw new Error(`something happened calling fromColumnValue`);
    }
    toColumnValue(prop, value) {
        /* istanbul ignore else */
        if (prop && prop[this.name] && prop[this.name].transform &&
            prop[this.name].transform.toDB) {
            return prop[this.name].transform.toDB(value);
        }
        /* istanbul ignore next */
        throw new Error(`something happened calling toColumnValue`);
    }
    // *************************************************************************************
    // Mapping:
    // -------------------------------------------------------------------------------------
    /**
     * get the default table/column name for a given model/property name
     * @param name The name
     */
    dbName(modelOrProperty) {
        return this.connector.dbName(modelOrProperty);
    }
    /**
     * escape/quote the table/column name
     * @param name The name
     */
    escapeName(dbName) {
        /* istanbul ignore if */
        if (!dbName) {
            return dbName;
        }
        return '`' + dbName.replace(/\`/g, '``').replace(/\./, '`.`') + '`';
    }
    /**
     * escape a value
     * @param name The name
    escapeValue(value: any): any {
      return value;
    }
     */
    // *************************************************************************************
    // pagination
    // -------------------------------------------------------------------------------------
    static buildLimit(filter) {
        const limit = filter.limit ? (isNaN(filter.limit) ? 0 : filter.limit) : 0;
        const offset = filter.offset ? (isNaN(filter.offset) ? 0 : filter.offset) : 0;
        /* istanbul ignore if */
        if (!limit && !offset) {
            return undefined;
        }
        const res = [];
        /* istanbul ignore else */
        if (limit) {
            res.push(`LIMIT ${limit}`);
        }
        if (offset) {
            res.push(`OFFSET ${offset}`);
        }
        return res.join(' ');
    }
    applyPagination(model, sql, filter) {
        const limitClause = Sqlite3JugglerConnector.buildLimit(filter);
        return limitClause ? sql.merge(limitClause) : sql;
    }
    // *************************************************************************************
    // automigration/autoupdate/isActual
    // -------------------------------------------------------------------------------------
    // TODO(?): add foreign_key_check to automigrate and autoupdate if foreign
    // keys are enabled?
    /**
     * Create the table for the given model
     * @param model The model name
     * @param  [cb] The callback function
     */
    createTable(model, cb) {
        const fn = () => __awaiter(this, void 0, void 0, function* () {
            try {
                const metaModel = this.getMetaModel(model, true);
                yield this.promisifiedExecuteSql(metaModel.table.getCreateTableStatement());
            }
            catch (err /* istanbul ignore next */) {
                return Promise.reject(err);
            }
            return;
        });
        return utils_1.callbackifyOrPromise(fn(), cb);
    }
    /**
     * drop the table for the given model
     * @param model The model name
     * @param  [cb] The callback function
     */
    dropTable(model, cb) {
        const fn = () => __awaiter(this, void 0, void 0, function* () {
            try {
                const metaModel = this.getMetaModel(model);
                yield this.promisifiedExecuteSql(metaModel.table.getDropTableStatement());
            }
            catch (err /* istanbul ignore next */) {
                return Promise.reject(err);
            }
            return;
        });
        return utils_1.callbackifyOrPromise(fn(), cb);
    }
    autoupdate(models, cb) {
        if (!cb && typeof models === 'function') {
            cb = models;
            models = undefined;
        }
        if (models && typeof models === 'string') {
            models = [models];
        }
        models = models || Object.keys(this._models);
        return this._autoupate(models, cb);
    }
    _autoupate(models, cb) {
        return utils_1.callbackifyOrPromise(this.promisifiedAutoupdate(models), cb);
    }
    promisifiedAutoupdate(models) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const tables = [];
                /* istanbul ignore else */
                if (models) {
                    models.forEach((model) => {
                        const metaModel = this.getMetaModel(model, true);
                        tables.push(metaModel.table);
                    });
                }
                const connection = yield this.connector.getConnection();
                const autoUpgrader = new sqlite3orm_1.AutoUpgrader(connection);
                yield autoUpgrader.upgradeTables(tables);
                try {
                    yield connection.close();
                }
                catch (_ignore) {
                }
            }
            catch (err) /* istanbul ignore next */ {
                return Promise.reject(err);
            }
        });
    }
    isActual(models, cb) {
        if (!cb && typeof models === 'function') {
            cb = models;
            models = undefined;
        }
        if (models && typeof models === 'string') {
            models = [models];
        }
        models = models || Object.keys(this._models);
        return this._isActual(models, cb);
    }
    _isActual(models, cb) {
        return utils_1.callbackifyOrPromise(this.promisifiedIsActual(models), cb);
    }
    promisifiedIsActual(models) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const tables = [];
                /* istanbul ignore else */
                if (models) {
                    models.forEach((model) => {
                        const metaModel = this.getMetaModel(model, true);
                        tables.push(metaModel.table);
                    });
                }
                const connection = yield this.connector.getConnection();
                const autoUpgrader = new sqlite3orm_1.AutoUpgrader(connection);
                const result = yield autoUpgrader.isActual(tables);
                try {
                    yield connection.close();
                }
                catch (_ignore) {
                }
                return result;
            }
            catch (err) /* istanbul ignore next */ {
                return Promise.reject(err);
            }
        });
    }
    // *************************************************************************************
    // TODO(?): discover model definition
    // -------------------------------------------------------------------------------------
    // discoverModelDefinitions
    // discoverModelProperties
    // discoverPrimaryKeys
    // discoverForeignKeys
    // discoverExportedForeignKeys
    // discoverSchema
    // discoverAndBuildModels
    // *************************************************************************************
    // model definitions
    // -------------------------------------------------------------------------------------
    getMetaModel(modelName, recreate) {
        const lbModelDef = this._models[modelName];
        /* istanbul ignore if */
        if (!lbModelDef) {
            throw new Error(`model '${modelName}' not found`);
        }
        return this.connector.getMetaModelFromJuggler(modelName, lbModelDef, recreate);
    }
    destroyMetaModel(modelName) {
        this.connector.destroyMetaModel(modelName);
    }
    destroyAllMetaModels() {
        this.connector.destroyAllMetaModels();
    }
    modelNames() {
        return this.connector.metaModels.modelNames();
    }
}
exports.Sqlite3JugglerConnector = Sqlite3JugglerConnector;
/**
 *
 * Initialize the Sqlite3Connector for the given data source
 *
 * @param dataSource The dataSource
 * @callback [cb] The callback function
 */
function initialize(dataSource, cb) {
    /* istanbul ignore if */
    if (dataSource.settings && !!dataSource.settings.debug) {
        sqlite3_connector_1.Sqlite3Connector.debugEnabled = true;
    }
    debug(`initialize ${sqlite3_connector_1.SQLITE3_CONNECTOR_NAME} juggler connector...`);
    const connector = new Sqlite3JugglerConnector(dataSource.settings);
    dataSource.connector = connector;
    dataSource.connector.dataSource = dataSource;
    /* istanbul ignore else */
    if (cb) {
        if (dataSource.settings && dataSource.settings.lazyConnect) {
            cb();
        }
        else {
            debug(`initial connect`);
            connector.connect((err) => {
                /* istanbul ignore if */
                if (err) {
                    debug(`initial connect failed: `, err);
                }
                else {
                    debug(`initial connect succeeded`);
                }
                debug(`initialized ${sqlite3_connector_1.SQLITE3_CONNECTOR_NAME} juggler connector`);
                cb(err);
            });
            return;
        }
    }
    debug(`initialized ${sqlite3_connector_1.SQLITE3_CONNECTOR_NAME} juggler connector`);
}
exports.initialize = initialize;
//# sourceMappingURL=sqlite3-juggler-connector.js.map